### A Pluto.jl notebook ###
# v0.16.1

using Markdown
using InteractiveUtils

# ╔═╡ ee082fcf-54d1-4aea-9944-8d7f81c6dbf4
using  BioCCP, Plots, PlutoUI, Distributions

# ╔═╡ 41beadc2-385e-42bf-9960-ab201242b400
md"*Loading the required packages...*"

# ╔═╡ 4d246460-af05-11eb-382b-590e60ba61f5
md"## BioCCP Case studies


In this notebook, the BioCCP.jl package will be applied to two real biological problems, based on two studies from literature. More specifically, we will 

(1) illustrate how BioCCP can aid in determining an appropriate sample size in order to guarantee sufficient coverage of a CRISPR-Cas 9 guide RNA library. For this case study, we consider the paper 'Genome-wide CRISPR Screen in a Mouse Model of Tumor Growth and Metastasis' (Chen *et al.*, 2015) [^1]. 

(2) show how BioCCP assists in sample determination for screening of modular proteins. For this case study, we will use the paper 'Rapid and High-Throughput Evaluation of Diverse Configurations of Engineered Lysins Using the VersaTile Technique' (Duyvejonck *et al.*, 2021) [^2].

"

# ╔═╡ ad7e5e06-55b2-4752-9335-2364489932eb
md"##### 1. [Genome-wide CRISPR Screen in a Mouse Model of Tumor Growth and Metastasis (Chen *et al*, 2015)](https://www.sciencedirect.com/science/article/pii/S0092867415002044)

The study focuses on a genome-wide CRISPR-Cas9 screen in a mouse model of tumor evolution as a powerful tool to identify loss-of-function mutations that correlate with tumor growth and metastasis.  

For this purpose, **over 60000 gRNAs** were designed, each targeting a protein-coding gene or microRNA precursor in the mouse genome. 
Each gRNA sequence was cloned into a lentiviral vector. From this plasmid pool, single gRNA viruses were generated by transfection of a HEK293FT cell line. Then, **cancer cells (Cas9 KPD cells) were infected with the single gRNA designed lentiviruses**. The number of lentiviruses, and by consequence, the number of integrated sgRNA constructs per cancer cell, is determined by a Poisson distribution (see later). The integrated sgRNA construct results in production of the corresponding gRNA by the infected cell, which guides the Cas9 enzyme to execute the knockout in the target gene. Afterwards, these **gRNA-producing cancer cells are transplanted into the flanks of mice**, causing primary tumor formation *in situ*. 

The question is **how many cells should be injected in total, so we can study the effect of each mutation on metastasis, or, how many cells must be transplanted into mice to observe all gRNAs at least once?**"

# ╔═╡ d6b7e9f5-3bda-4477-b92c-e32b836f1f0d
md"##### 1.1 Problem definition"

# ╔═╡ 50462b1a-f65e-4d91-8d8d-da9c93ad007c
Show(MIME"image/png"(), read("CRISPR_img.png"))

# ╔═╡ f3ef2715-da53-449e-b198-faeeb78ac83f
md"###### 🔹 Number of modules (*n*)

In BioCCP.jl terminology, we **consider each gRNA as a module**. There are 63090 gRNAs present in the gRNA plasmid library (*i.e.*, the number of gRNAs with more than 1 read as uncovered by deep sequencing the plasmid library (see below))."

# ╔═╡ 08054214-4045-486a-ac44-7a1a8aee9acf
n_gRNAs = 63090

# ╔═╡ 50d87809-df3b-4558-97ce-0a50fdfbcf69
md"###### 🔹 Module distribution (*p*)

The gRNA sequences were embedded into a plasmid library, with a single gRNA per lentiviral vector. Deep sequencing of this plasmid pool resulted in a number of reads for each gRNA, representing the abundance of each gRNA in the plasmid pool:

*Note: The log normalized reads were retrieved from the supplementary data accompanying the paper and converted to normalized reads for each gRNA.*"

# ╔═╡ 857da523-7c09-4230-9397-2dc0ef639007
reads_gRNA = [7.33862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
7.333862316
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
10.50079347
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
13.66772463
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
16.83465579
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
20.00158695
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
23.16851811
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
26.33544927
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
29.50238042
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
32.66931158
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
35.83624274
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
39.0031739
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
42.17010506
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
45.33703621
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
48.50396737
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
51.67089853
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
54.83782969
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
58.00476085
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
61.17169201
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
64.33862316
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
67.50555432
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
70.67248548
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
73.83941664
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
77.0063478
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
80.17327896
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
83.34021011
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
86.50714127
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
89.67407243
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
92.84100359
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
96.00793475
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
99.1748659
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
102.3417971
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
105.5087282
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
108.6756594
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
111.8425905
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
115.0095217
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
118.1764529
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
121.343384
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
124.5103152
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
127.6772463
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
130.8441775
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
134.0111086
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
137.1780398
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
140.344971
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
143.5119021
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
146.6788333
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
149.8457644
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
153.0126956
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
156.1796268
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
159.3465579
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
162.5134891
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
165.6804202
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
168.8473514
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
172.0142825
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
175.1812137
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
178.3481449
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
181.515076
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
184.6820072
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
187.8489383
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
191.0158695
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
194.1828007
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
197.3497318
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
200.516663
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
203.6835941
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
206.8505253
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
210.0174564
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
213.1843876
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
216.3513188
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
219.5182499
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
222.6851811
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
225.8521122
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
229.0190434
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
232.1859745
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
235.3529057
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
238.5198369
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
241.686768
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
244.8536992
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
248.0206303
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
251.1875615
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
254.3544927
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
257.5214238
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
260.688355
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
263.8552861
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
267.0222173
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
270.1891484
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
273.3560796
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
276.5230108
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
279.6899419
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
282.8568731
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
286.0238042
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
289.1907354
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
292.3576666
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
295.5245977
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
298.6915289
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
301.85846
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
305.0253912
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
308.1923223
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
311.3592535
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
314.5261847
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
317.6931158
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
320.860047
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
324.0269781
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
327.1939093
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
330.3608405
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
333.5277716
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
336.6947028
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
339.8616339
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
343.0285651
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
346.1954962
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
349.3624274
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
352.5293586
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
355.6962897
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
358.8632209
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
362.030152
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
365.1970832
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
368.3640144
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
371.5309455
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
374.6978767
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
377.8648078
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
381.031739
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
384.1986701
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
387.3656013
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
390.5325325
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
393.6994636
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
396.8663948
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
400.0333259
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
403.2002571
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
406.3671882
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
409.5341194
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
412.7010506
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
415.8679817
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
419.0349129
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
422.201844
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
425.3687752
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
428.5357064
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
431.7026375
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
434.8695687
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
438.0364998
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
441.203431
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
444.3703621
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
447.5372933
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
450.7042245
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
453.8711556
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
457.0380868
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
460.2050179
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
463.3719491
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
466.5388803
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
469.7058114
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
472.8727426
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
476.0396737
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
479.2066049
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
482.373536
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
485.5404672
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
488.7073984
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
491.8743295
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
495.0412607
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
498.2081918
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
501.375123
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
504.5420542
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
507.7089853
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
510.8759165
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
514.0428476
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
517.2097788
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
520.3767099
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
523.5436411
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
526.7105723
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
529.8775034
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
533.0444346
536.2113657
536.2113657
536.2113657
536.2113657
536.2113657
536.2113657
536.2113657
536.2113657
536.2113657
536.2113657
536.2113657
536.2113657
536.2113657
536.2113657
536.2113657
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
539.3782969
542.5452281
542.5452281
542.5452281
542.5452281
542.5452281
542.5452281
542.5452281
542.5452281
542.5452281
542.5452281
542.5452281
542.5452281
542.5452281
542.5452281
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
545.7121592
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
548.8790904
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
552.0460215
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
555.2129527
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
558.3798838
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
561.546815
564.7137462
564.7137462
564.7137462
564.7137462
564.7137462
564.7137462
564.7137462
564.7137462
564.7137462
564.7137462
564.7137462
564.7137462
564.7137462
564.7137462
564.7137462
564.7137462
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
567.8806773
571.0476085
571.0476085
571.0476085
571.0476085
571.0476085
571.0476085
571.0476085
571.0476085
571.0476085
571.0476085
571.0476085
571.0476085
571.0476085
571.0476085
571.0476085
571.0476085
571.0476085
571.0476085
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
574.2145396
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
577.3814708
580.548402
580.548402
580.548402
580.548402
580.548402
580.548402
580.548402
580.548402
580.548402
580.548402
580.548402
580.548402
580.548402
580.548402
580.548402
580.548402
580.548402
580.548402
583.7153331
583.7153331
583.7153331
583.7153331
583.7153331
583.7153331
583.7153331
583.7153331
583.7153331
583.7153331
583.7153331
583.7153331
583.7153331
583.7153331
583.7153331
583.7153331
586.8822643
586.8822643
586.8822643
586.8822643
586.8822643
586.8822643
586.8822643
586.8822643
586.8822643
586.8822643
586.8822643
586.8822643
586.8822643
586.8822643
586.8822643
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
590.0491954
593.2161266
593.2161266
593.2161266
593.2161266
593.2161266
593.2161266
593.2161266
593.2161266
593.2161266
593.2161266
593.2161266
593.2161266
593.2161266
596.3830577
596.3830577
596.3830577
596.3830577
596.3830577
596.3830577
596.3830577
596.3830577
596.3830577
596.3830577
599.5499889
599.5499889
599.5499889
599.5499889
599.5499889
599.5499889
599.5499889
599.5499889
599.5499889
599.5499889
599.5499889
599.5499889
599.5499889
599.5499889
599.5499889
599.5499889
599.5499889
602.7169201
602.7169201
602.7169201
602.7169201
602.7169201
602.7169201
602.7169201
602.7169201
602.7169201
602.7169201
602.7169201
602.7169201
602.7169201
602.7169201
602.7169201
602.7169201
602.7169201
605.8838512
605.8838512
605.8838512
605.8838512
605.8838512
605.8838512
605.8838512
605.8838512
605.8838512
605.8838512
605.8838512
605.8838512
605.8838512
605.8838512
605.8838512
605.8838512
605.8838512
609.0507824
609.0507824
609.0507824
609.0507824
609.0507824
609.0507824
609.0507824
609.0507824
609.0507824
609.0507824
609.0507824
609.0507824
609.0507824
609.0507824
609.0507824
612.2177135
612.2177135
612.2177135
612.2177135
612.2177135
612.2177135
612.2177135
612.2177135
612.2177135
612.2177135
612.2177135
612.2177135
612.2177135
612.2177135
615.3846447
615.3846447
615.3846447
615.3846447
615.3846447
615.3846447
615.3846447
615.3846447
615.3846447
615.3846447
618.5515758
618.5515758
618.5515758
618.5515758
618.5515758
618.5515758
618.5515758
618.5515758
618.5515758
618.5515758
618.5515758
618.5515758
618.5515758
618.5515758
618.5515758
621.718507
621.718507
621.718507
621.718507
621.718507
621.718507
621.718507
621.718507
621.718507
621.718507
624.8854382
624.8854382
624.8854382
628.0523693
628.0523693
628.0523693
628.0523693
628.0523693
628.0523693
628.0523693
628.0523693
628.0523693
628.0523693
631.2193005
631.2193005
631.2193005
631.2193005
631.2193005
631.2193005
631.2193005
631.2193005
631.2193005
631.2193005
631.2193005
631.2193005
631.2193005
634.3862316
634.3862316
634.3862316
634.3862316
634.3862316
634.3862316
634.3862316
634.3862316
634.3862316
634.3862316
634.3862316
634.3862316
634.3862316
634.3862316
634.3862316
634.3862316
637.5531628
637.5531628
637.5531628
637.5531628
637.5531628
637.5531628
637.5531628
637.5531628
637.5531628
637.5531628
640.720094
640.720094
640.720094
640.720094
643.8870251
643.8870251
643.8870251
643.8870251
643.8870251
643.8870251
643.8870251
643.8870251
643.8870251
647.0539563
647.0539563
647.0539563
647.0539563
647.0539563
647.0539563
647.0539563
647.0539563
650.2208874
650.2208874
650.2208874
650.2208874
650.2208874
650.2208874
650.2208874
650.2208874
650.2208874
650.2208874
653.3878186
653.3878186
653.3878186
656.5547497
656.5547497
656.5547497
656.5547497
656.5547497
656.5547497
656.5547497
656.5547497
656.5547497
656.5547497
656.5547497
659.7216809
659.7216809
659.7216809
659.7216809
659.7216809
659.7216809
659.7216809
659.7216809
659.7216809
659.7216809
659.7216809
662.8886121
662.8886121
662.8886121
662.8886121
662.8886121
666.0555432
666.0555432
666.0555432
666.0555432
666.0555432
666.0555432
666.0555432
666.0555432
669.2224744
669.2224744
672.3894055
672.3894055
672.3894055
672.3894055
672.3894055
672.3894055
672.3894055
672.3894055
675.5563367
675.5563367
675.5563367
675.5563367
678.7232679
678.7232679
678.7232679
678.7232679
678.7232679
678.7232679
678.7232679
678.7232679
678.7232679
678.7232679
681.890199
681.890199
681.890199
681.890199
681.890199
681.890199
685.0571302
685.0571302
685.0571302
685.0571302
685.0571302
688.2240613
688.2240613
688.2240613
688.2240613
688.2240613
688.2240613
688.2240613
688.2240613
688.2240613
688.2240613
691.3909925
691.3909925
691.3909925
694.5579236
694.5579236
694.5579236
694.5579236
697.7248548
697.7248548
697.7248548
697.7248548
697.7248548
700.891786
700.891786
700.891786
700.891786
700.891786
700.891786
704.0587171
704.0587171
704.0587171
704.0587171
704.0587171
704.0587171
704.0587171
707.2256483
707.2256483
710.3925794
710.3925794
710.3925794
713.5595106
716.7264418
716.7264418
716.7264418
719.8933729
719.8933729
719.8933729
719.8933729
719.8933729
719.8933729
723.0603041
726.2272352
726.2272352
726.2272352
729.3941664
729.3941664
729.3941664
729.3941664
732.5610975
732.5610975
732.5610975
732.5610975
732.5610975
732.5610975
735.7280287
738.8949599
738.8949599
738.8949599
738.8949599
738.8949599
742.061891
745.2288222
745.2288222
745.2288222
745.2288222
748.3957533
748.3957533
748.3957533
748.3957533
748.3957533
748.3957533
751.5626845
751.5626845
754.7296157
761.063478
761.063478
761.063478
761.063478
764.2304091
764.2304091
764.2304091
764.2304091
764.2304091
767.3973403
770.5642714
770.5642714
770.5642714
773.7312026
773.7312026
776.8981338
780.0650649
780.0650649
783.2319961
783.2319961
786.3989272
786.3989272
786.3989272
786.3989272
789.5658584
789.5658584
789.5658584
789.5658584
792.7327896
792.7327896
792.7327896
792.7327896
795.8997207
795.8997207
799.0666519
799.0666519
802.233583
802.233583
805.4005142
808.5674453
808.5674453
811.7343765
811.7343765
814.9013077
814.9013077
818.0682388
821.23517
821.23517
821.23517
824.4021011
830.7359634
830.7359634
837.0698258
840.2367569
840.2367569
846.5706192
849.7375504
849.7375504
849.7375504
849.7375504
852.9044816
856.0714127
859.2383439
859.2383439
862.405275
862.405275
862.405275
868.7391373
871.9060685
878.2399308
878.2399308
890.9076555
897.2415178
906.7423112
913.0761736
913.0761736
916.2431047
916.2431047
922.576967
932.0777605
947.9124163
947.9124163
957.4132098
963.7470721
963.7470721
966.9140033
970.0809344
973.2478656
973.2478656
979.5817279
985.9155902
1023.918764
1036.586489
1049.254213
1068.2558
1074.589663
1080.923525
1099.925112
1131.594423
1144.262148
1258.27167
1372.281192
]

# ╔═╡ 737c8375-2c49-4f32-b54c-1f15f8d451d5
md"which can be summarized in the following unimodal read distribution: "

# ╔═╡ 94bcc8de-a0be-47ab-a03a-b04c351ad6f0
begin
	histogram(reads_gRNA, bar_edges=false, bins=100,  size = (700, 320), orientation=:v, titlefont=font(10), xguidefont=font(9), yguidefont=font(9), label="")
	xlabel!("Number of reads")
	ylabel!("Density")
	title!("Distribution of gRNA reads in plasmid pool")
end

# ╔═╡ 364c38ca-8637-4d26-8d64-525222d24033
md"Normalizing for the total number of reads in the plasmid pool gives us a **probability vector** *p*, **containing for each gRNA a probability to be randomly sampled from the plasmid library** (proportional to its abundance in the library):"

# ╔═╡ 5841ea1a-8d23-4fe7-8010-7d8512050c22
p_gRNA = reads_gRNA / sum(reads_gRNA)

# ╔═╡ e1444e8d-c0ae-408f-ab88-fd11459d9842
md"###### 🔹 Number of modules per design (*r*)
The single gRNA lentiviruses are used to infect cancer cells (Cas9 KPD cells). The Multiplicity of Infection (MOI), *i.e.*, the number of virions added per cell, amounts to 0.4: "

# ╔═╡ 5c906c4a-8785-4199-8aba-a35c855e77f6
MOI = 0.4

# ╔═╡ 449ad144-90d4-4f74-867f-939b489ddaad
md"The probability of a cell being infected by $k$ viral particles (= containing $k$ integrated RNA constructs) can be described by a Poisson distribution with λ = MOI:

$$P(k) = \frac{λ^k . e^{-λ}}{k!}$$"

# ╔═╡ 28dda5a1-546a-411f-9b8b-398e37195b13
λ = MOI

# ╔═╡ aad92f19-6935-4cbd-8e0f-d16a0d379386
poisson = Poisson(λ)

# ╔═╡ 69fe641d-afb9-45dc-9aa0-436177b02ac3
p_0gRNA = pdf(poisson, 0) # the probability that a cancer cell has 0 integrated gRNA constructs

# ╔═╡ e88e908f-be65-488f-a0dd-718a091f9d5c
p_1gRNA = pdf(poisson, 1) # the probability that a cancer cell has 1 integrated gRNA construct

# ╔═╡ 480d6282-ce97-4127-957d-d0b9fd37a7e4
p_2gRNA = pdf(poisson, 2) # the probability that a cancer cell has 2 integrated gRNA  constructs

# ╔═╡ 55e0b804-a3ee-4667-8367-38b1b20b0096
md"
After selection on antibiotics, only cells that have received at least one sgRNA-expressing lentiviral integrant survive. The percentage of cells surviving can be written as:

$$P_{survival} = P(k>0) = 1-P(k=0)$$

"

# ╔═╡ d021a50b-3bd1-4125-a939-1bba5eace898
Pₛᵤᵣᵥᵢᵥₐₗ = 1 - pdf(poisson, 0)

# ╔═╡ 2bedff06-473e-49c0-9d8c-a0cde03ad554
md"With this information, we can calculate the **average number of gRNA integrands per cancer cell** *r*:"

# ╔═╡ fa7b1373-006c-48e8-917b-c750bce86e09
r = sum([k*pdf(poisson, k)/Pₛᵤᵣᵥᵢᵥₐₗ for k in 1:20]) 

# calculation of expected value: summation over number of integrated constructs multiplied by its probability (normalized by percentage of cells that survived)

# ╔═╡ 2ac080a7-af56-41ae-b913-8d65dedb9197


# ╔═╡ b4cc09e0-2a6a-4ab6-8153-cbc817dede2e
md"##### 1.2   Statistics for determining a minimum sample size using BioCCP.jl
"

# ╔═╡ 3466f636-7b06-42e0-b71e-55a7a2c74808
md"###### 🔹 Expected minimum number of cells 

The **expected minimum number of cells** that need be injected into the mice so  each gRNA is present in the experiment at least once (*m = 1*):"

# ╔═╡ 0a99939a-4edb-488d-bbdf-4a425c808607
m = 1

# ╔═╡ f35928d5-e8c2-4c9f-b4ba-9cb58eafb9e0
expectation_minsamplesize(n_gRNAs; p = p_gRNA, r = r, m = m)

# ╔═╡ b8fe484e-1189-4fd0-85b9-715e084d65da
std_minsamplesize(n_gRNAs; p = p_gRNA, r = r, m = m)

# ╔═╡ d4cd5ff7-88b8-4e5c-a540-04c51ad51f42


# ╔═╡ 224d556c-cd9d-4c00-9cde-1c3df7cfcdef
md"In the study of Chen *et al.*, 3 × 10⁷ cells are injected into the flanks of mice. This exceeds the expected minimum sample size to observe each gRNA at least once."

# ╔═╡ aa29947a-4dcb-48d8-9ad0-3828f8d4b8d2
sample_size_paper = 3*10^7

# ╔═╡ a27384c0-db19-4b06-8165-208d3fe77350
md"###### 🔹 Probability to observe each gRNA at least once w.r.t. sample size"

# ╔═╡ 7b43e546-07f0-45ce-9d2a-52894adad822
md"The **success probability curve**, describing the probability that the minimum number of cells that should be sampled to observe all gRNAs at least $m time(s) is smaller than or equal to a given sample size  on the x-axis:"

# ╔═╡ ca7ecf8b-634e-4fa5-aaa8-102fa488c7a1
begin
	sample_size_initial = 18*10^6
		while (1 - success_probability(n_gRNAs, sample_size_initial; p = p_gRNA, r = r, m = m)) > 0.00005
			global sample_size_initial += Int(ceil(10*n_gRNAs))
		end
			
		sample_sizes = Int.(ceil.(0:n_gRNAs*5:sample_size_initial))
		successes = success_probability.(n_gRNAs, sample_sizes; p = p_gRNA, r = r, m = m)
	plot(sample_sizes, successes, title = "Success probability to sample each gRNA at least $m time 
in function of sample size", xlabel = "number of cells sampled", ylabel= "success probability", label = "", legend=:bottomright, size=(600,400), seriestype=:scatter, titlefont=font(10),xguidefont=font(9), yguidefont=font(9))
end

# ╔═╡ 60c3ce0d-85be-42f0-a869-1895abc096f3
md"When the number of cells is equal to $sample_size_paper, the probability that all gRNAs will be represented in the genome-wide screening experiment at least once, is:"

# ╔═╡ 7eb18559-e2c0-4c34-a2b2-4e3c3ab831a6
success_probability(63090, 3*10^7; p = p_gRNA, r = r, m = m)

# ╔═╡ ecf6292c-7cdd-4886-8a59-be8e4f2751b7
md"Let's investigate how many complete sets of gRNAs that are suffficiently covered by the sample size of the study by Chen *et al.*. Let's define sufficient coverage as a success probability of at least 95%."

# ╔═╡ 56f21296-fcd2-46f7-a130-4cb6940c1133
success_probability(63090, sample_size_paper; p = p_gRNA, r = r, m = 2)

# ╔═╡ 09d77a14-01f4-430c-9038-25b85b31a1c2
success_probability(63090, sample_size_paper; p = p_gRNA, r = r, m = 7)

# ╔═╡ 071cdabe-3443-490c-ab79-e270a0a1de68
success_probability(63090, sample_size_paper; p = p_gRNA, r = r, m = 10)

# ╔═╡ c59c5fb7-7291-46e2-88af-03ed38c7c3eb
begin
	success_probabilities = []
		for m in 1:15
			push!(success_probabilities, success_probability(63090, sample_size_paper; p = p_gRNA, r = r, m = m))
	end
	
	plot(success_probabilities, seriestype=:scatter, 
		title="Coverage for sample size of $sample_size_paper cells", 
		xlabel="number of complete sets of gRNAs to observe (m)", ylabel="success probability", label="")
	plot!([0, 15], [0.95, 0.95], label="95% success
probability", legend = :best, titlefont=font(10),xguidefont=font(9), yguidefont=font(9))
end

# ╔═╡ bb990e0c-df90-4635-80e4-e6639a62865a
md"We see the sample size of the paper covers 9 complete sets of gRNAs. Thereafter, the success probability drops below 95%."

# ╔═╡ 57309e16-8c1b-4268-b164-0b45b60b4fbd


# ╔═╡ ca947d4b-18e6-4bad-8d9b-b73324a5c40b
md"###### 🔹 Expected fraction of gRNAs observed w.r.t. sample size"

# ╔═╡ dc696281-7a5b-4568-a4c2-8dde90af43f0
md"The **fraction of the total number of available gRNAs that is expected to be observed** after injecting a given number of cells, is displayed by the curve below:"

# ╔═╡ 7968de5e-5ae8-4ab4-b089-c3d33475af2f
begin
global sample_size_initial_frac = 7*10^6
		while (1 - expectation_fraction_collected(n_gRNAs, sample_size_initial_frac; p=p_gRNA, r = r)) > 0.000005
		global	 sample_size_initial_frac += Int(ceil(10*n_gRNAs))
		end
	
	sample_sizes_frac = Int.(ceil.(collect(0 : n_gRNAs/2 : sample_size_initial_frac)))
	fracs = expectation_fraction_collected.(n_gRNAs, sample_sizes_frac; p=p_gRNA, r = r)
	
	plot(sample_sizes_frac, fracs, 
		title = "Expected observed fraction of the total number of gRNAs", 
	    xlabel = "number of cells", seriestype=:scatter, 
	    ylabel= "Expected observed fraction of gRNAs", 
		label = "", 
		size=(700,400), xguidefont=font(9), yguidefont=font(9), titlefont=font(10))
 	
end

# ╔═╡ be8e6332-f79d-4d63-afae-51c2d829f998
md"When 50000 cells are injected, it is expected that, on average, approximately 53% of the gRNAs is represented:"

# ╔═╡ f0eaf96b-0bc0-4194-9a36-886cb1d66e00
expectation_fraction_collected(n_gRNAs, 5*10^4; p = p_gRNA, r = r)

# ╔═╡ 293386f0-aafe-4e94-8e16-3237659d6963


# ╔═╡ b2111d05-a153-4969-aeae-a7f0c01e3365
md"###### 🔹 Occurence of specific gRNA w.r.t. sample size" 

# ╔═╡ 667fc6a8-b3ff-464d-903c-e215e7d2f472
pᵢ_gRNA = p_gRNA[1]

# ╔═╡ ead64d36-947a-4b9f-a0f7-a1821039f5b3
n_cells = 4*10^6

# ╔═╡ f92a6b6e-a556-45cb-a1ae-9f5fe791ffd2
md""" For the gRNA with the lowest abundance in the plasmid library, the probability that it is observed *k* times when injecting $n_cells cancer cells in the mice, can be described by the following curve:"""

# ╔═╡ 60fff6ab-3e19-4af7-b102-17d3d47494f3
begin
	ed = Int(floor(n_cells*pᵢ_gRNA*r))
	j = collect(0:1:2*ed)
	x  = prob_occurrence_module.(pᵢ_gRNA, n_cells, r, j)
	plot(j,x, seriestype=[:scatter, :line], xlabel="number of times gRNA is represented",
					ylabel="probability", 
					title=
		"Probability that gRNA is represented k times in mouse model
when $n_cells cells injected", 
					label="",
					size=((600,300)),titlefont=font(10), 
					xguidefont=font(9), yguidefont=font(9))	 
end

# ╔═╡ a041652b-365e-4594-9c48-c63d547b3295
mean, std = floor(n_cells * pᵢ_gRNA * r), floor(sqrt(n_cells * pᵢ_gRNA * r))

# ╔═╡ cc4c712d-6562-4d25-8b84-64458cda4198
md"The least abundant gRNA will be present on average 3 times with a standard deviation of 1, when 4x10⁶ cells are injected."

# ╔═╡ 752392c9-198a-4d5a-8ce7-7c154ce834ec


# ╔═╡ 1f2c0903-13af-48d5-bbf5-b6102043a288


# ╔═╡ a6014a30-6643-4504-8081-17bcc8be2615
md"##### 2. [Rapid and High-Throughput Evaluation of Diverse Configurations of Engineered Lysins Using the VersaTile Technique (Duyvejonck *et al.*, 2021)](https://www.mdpi.com/2079-6382/10/3/293)

This study aims engineering **modular endolysins** for obtaining optimal antibacterial properties. 

###### 2.1 Problem definition

The endolysin engineered in the study can consists of different protein domains, e.g.:

- an outer membrane permeabilizing peptide (OMP),
- an enzymatically active domain (EAD),
- a cell wall binding domain (CBD),
- a peptide linker

Different variants of these domain types are available and combined with the VersaTile DNA assembly technique.  The following picture represents an endolysin configuration that is considered in the paper:"

# ╔═╡ 45fd4574-679f-42b8-9aa5-b949d252e2e7
Show(MIME"image/png"(), read("Endolysin_img.png"))

# ╔═╡ 8032bbab-67a8-4058-873a-a33c4b39066f
md"In this library, there is an availability of 

- 42 OMPs, 
- 7 linkers,
- 6 CBDs, and
- 23 EADs.  

This results in a total number of 42 x 7 x 6 x 23 = **40572 possible endolysins**."

# ╔═╡ 5f480d13-2dd1-451e-adf5-45d5d7749fdc
begin
	n_OMP = 42;
	n_linker = 7;
	n_CBD = 6;
	n_EAD = 23;
end

# ╔═╡ da16c262-4d76-4757-9e80-df6871546278
md"However, due to limitations in the screening technique, only **188 endolysins from this library were analysed** for their activity against *Klebsiella pneumoniae*. "

# ╔═╡ 8b0ca97d-e7fd-4720-80a9-87c43c1f78a9


# ╔═╡ a4e1a017-c37c-4a81-915c-96638964eda2
md"###### 2.2 Determining coverage using BioCCP.jl"

# ╔═╡ 12afa1a7-b4ed-4767-aff8-07dab6fa5d3c
endolysins_sample_size = 188

# ╔═╡ 99d3250d-2335-425e-b470-2f570cba7367


# ╔═╡ dad9f7f9-a5b0-44b2-b1a9-96b0fe9b3c7c
md"###### 🔹 Coverage OMP variants in endolysin library"

# ╔═╡ 16fd6703-d559-444c-8f42-26e314f29fb3
md"First, let's take a look at the coverage of the OMP variants in the screening experiment. The **probability that all available OMPs were observed at least once in this set**, will be calculated below. We assume that there is an equal probability for each OMP to be observed (optimistic scenario)."

# ╔═╡ 63eed55a-1dfe-4b3d-b7aa-e4736718b105
Float16(success_probability(n_OMP, endolysins_sample_size; p = ones(n_OMP)/n_OMP, m = 1))

# ╔═╡ 55631080-6d7d-4059-b8c1-32fa10ef8884
md"The OMPs are only guaranteed to be fully covered with 62 % probability at a sample size of 188 endolysins. "

# ╔═╡ efac7679-df64-4db2-8b8b-06febdc8a5f0
md"The success probability is way lower than 95%. Suppose we would execute a series of experiments in which we sample endolysins until all OMPs are observed at least once, what would be the **average required number of endolysins per experiment to see each OMP at least once** in the set of sampled endolysins?"

# ╔═╡ d26ab92f-3541-4eb6-9257-124266d6878a
expectation_minsamplesize(n_OMP; p = ones(n_OMP)/n_OMP, m = 1)

# ╔═╡ 0f98810b-c6c5-4f4b-9c40-0af0ec78c057
std_minsamplesize(n_OMP; p = ones(n_OMP)/n_OMP, m = 1)

# ╔═╡ 7a1bed30-ecb8-4747-bf83-578b77f48e59
md"The expected minimum number of endolysins to observe each OMP variant at least once is 182 (standard deviation: 51 endolysins)."

# ╔═╡ c0ccc9c2-278e-4c21-ac1f-53c73d209e38
md"Another research question we can ask ourselves is what **fraction of the total number of OMP variants is expected to be represented** for the set of 188 randomly sampled endolysins from the library."

# ╔═╡ 908644a9-a4db-41f7-9faf-1f3afb5dac79
expectation_fraction_collected(n_OMP, endolysins_sample_size; p = ones(n_OMP)/n_OMP)

# ╔═╡ 19f4c16d-2f59-45ba-8502-b86cacd9cd10
md"In other words, when over different sampling experiments, each time 188 endolysins are randomly sampled, on average a fraction of 98.9% of all OMP variants is expected to be observed."

# ╔═╡ ae2c1a89-cff9-4514-8744-f1620e46663d


# ╔═╡ feedcbb2-e6a0-4cb6-9d88-81554ba4ccbf
md"###### 🔹 Coverage linkers, CBDs and EADs in endolysin library"

# ╔═╡ a34b71fb-1da6-43db-981b-71e1a404e4d6
md"Now, let's investigate how well the other module types are covered. The **probability that all available linkers were observed at least once in this set** (assuming that there is an equal probability for each linker to be observed):"

# ╔═╡ ca9f4254-b38c-4de2-bafb-f97dd15a46bc
Float16(success_probability(n_linker, endolysins_sample_size; p = ones(n_linker)/n_linker))

# ╔═╡ cf19ee57-174d-42b9-8f9d-f4654907fc2a
md"The **probability that all available CBDs were observed at least once in this set** (assuming that there is an equal probability for each CBD to be observed):"

# ╔═╡ 63d90d97-ba80-449e-8d38-1bba52ab32ac
Float16(success_probability(n_CBD, endolysins_sample_size; p = ones(n_CBD)/n_CBD))

# ╔═╡ 062ceeb7-5d6d-48ce-8ad5-9e1d78a937e7
md"The **probability that all available EADs were observed at least once in this set** (assuming that there is an equal probability for each EAD to be observed):"

# ╔═╡ 04a2acf6-52e2-42d7-96a0-f3e31d1f6b58
Float16(success_probability(n_EAD, endolysins_sample_size; p = ones(n_EAD)/n_EAD))

# ╔═╡ 63dedbcb-7ea0-4c4e-bb4c-fa8e258f9a93
md"We can conclude the EADs, CBDs and linker domains were sufficiently covered by the sample size of $endolysins_sample_size endolysins."

# ╔═╡ fbffaab6-3154-49df-a226-d5810d0b7c38
md"""## References"""

# ╔═╡ 1f48143a-2152-4bb9-a765-a25e70c281a3
md"""[^1]:  Chen, S., Sanjana, N. E., Zheng, K., Shalem, O., Lee, K., Shi, X., ... & Sharp, P. A. (2015). *Genome-wide CRISPR screen in a mouse model of tumor growth and metastasis.* Cell, 160(6), 1246-1260.

[^2]: Duyvejonck, L., Gerstmans, H., Stock, M., Grimon, D., Lavigne, R., & Briers, Y. (2021). *Rapid and High-Throughput Evaluation of Diverse Configurations of Engineered Lysins Using the VersaTile Technique.* Antibiotics, 10(3), 293.

"""


# ╔═╡ 00000000-0000-0000-0000-000000000001
PLUTO_PROJECT_TOML_CONTENTS = """
[deps]
BioCCP = "79e6b149-e254-49fe-a721-3c4960de1574"
Distributions = "31c24e10-a181-5473-b8eb-7969acd0382f"
Plots = "91a5bcdd-55d7-5caf-9e0b-520d859cae80"
PlutoUI = "7f904dfe-b85e-4ff6-b463-dae2292396a8"

[compat]
BioCCP = "~0.1.0"
Distributions = "~0.25.18"
Plots = "~1.22.4"
PlutoUI = "~0.7.16"
"""

# ╔═╡ 00000000-0000-0000-0000-000000000002
PLUTO_MANIFEST_TOML_CONTENTS = """
# This file is machine-generated - editing it directly is not advised

[[Adapt]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "84918055d15b3114ede17ac6a7182f68870c16f7"
uuid = "79e6a3ab-5dfb-504d-930d-738a2a938a0e"
version = "3.3.1"

[[ArgTools]]
git-tree-sha1 = "bdf73eec6a88885256f282d48eafcad25d7de494"
uuid = "0dad84c5-d112-42e6-8d28-ef12dabb789f"
version = "1.1.1"

[[Artifacts]]
deps = ["Pkg"]
git-tree-sha1 = "c30985d8821e0cd73870b17b0ed0ce6dc44cb744"
uuid = "56f22d72-fd6d-98f1-02f0-08ddc0907c33"
version = "1.3.0"

[[Base64]]
uuid = "2a0f44e3-6c83-55bd-87e4-b1978d98bd5f"

[[BioCCP]]
deps = ["Distributions"]
git-tree-sha1 = "791b4218060972fa3a37313208c674597581ee05"
uuid = "79e6b149-e254-49fe-a721-3c4960de1574"
version = "0.1.0"

[[Bzip2_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "c3598e525718abcc440f69cc6d5f60dda0a1b61e"
uuid = "6e34b625-4abd-537c-b88f-471c36dfa7a0"
version = "1.0.6+5"

[[Cairo_jll]]
deps = ["Artifacts", "Bzip2_jll", "Fontconfig_jll", "FreeType2_jll", "Glib_jll", "JLLWrappers", "LZO_jll", "Libdl", "Pixman_jll", "Pkg", "Xorg_libXext_jll", "Xorg_libXrender_jll", "Zlib_jll", "libpng_jll"]
git-tree-sha1 = "e2f47f6d8337369411569fd45ae5753ca10394c6"
uuid = "83423d85-b0ee-5818-9007-b63ccbeb887a"
version = "1.16.0+6"

[[ChainRulesCore]]
deps = ["Compat", "LinearAlgebra", "SparseArrays"]
git-tree-sha1 = "a325370b9dd0e6bf5656a6f1a7ae80755f8ccc46"
uuid = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
version = "1.7.2"

[[ColorSchemes]]
deps = ["ColorTypes", "Colors", "FixedPointNumbers", "Random"]
git-tree-sha1 = "a851fec56cb73cfdf43762999ec72eff5b86882a"
uuid = "35d6a980-a343-548e-a6ea-1d62b119f2f4"
version = "3.15.0"

[[ColorTypes]]
deps = ["FixedPointNumbers", "Random"]
git-tree-sha1 = "024fe24d83e4a5bf5fc80501a314ce0d1aa35597"
uuid = "3da002f7-5984-5a60-b8a6-cbb66c0b333f"
version = "0.11.0"

[[Colors]]
deps = ["ColorTypes", "FixedPointNumbers", "Reexport"]
git-tree-sha1 = "417b0ed7b8b838aa6ca0a87aadf1bb9eb111ce40"
uuid = "5ae59095-9a9b-59fe-a467-6f913c188581"
version = "0.12.8"

[[Compat]]
deps = ["Base64", "Dates", "DelimitedFiles", "Distributed", "InteractiveUtils", "LibGit2", "Libdl", "LinearAlgebra", "Markdown", "Mmap", "Pkg", "Printf", "REPL", "Random", "SHA", "Serialization", "SharedArrays", "Sockets", "SparseArrays", "Statistics", "Test", "UUIDs", "Unicode"]
git-tree-sha1 = "31d0151f5716b655421d9d75b7fa74cc4e744df2"
uuid = "34da2185-b29b-5c13-b0c7-acf172513d20"
version = "3.39.0"

[[CompilerSupportLibraries_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "8e695f735fca77e9708e795eda62afdb869cbb70"
uuid = "e66e0078-7015-5450-92f7-15fbd957f2ae"
version = "0.3.4+0"

[[Contour]]
deps = ["StaticArrays"]
git-tree-sha1 = "9f02045d934dc030edad45944ea80dbd1f0ebea7"
uuid = "d38c429a-6771-53c6-b99e-75d170b6e991"
version = "0.5.7"

[[DataAPI]]
git-tree-sha1 = "cc70b17275652eb47bc9e5f81635981f13cea5c8"
uuid = "9a962f9c-6df0-11e9-0e5d-c546b8b5ee8a"
version = "1.9.0"

[[DataStructures]]
deps = ["Compat", "InteractiveUtils", "OrderedCollections"]
git-tree-sha1 = "7d9d316f04214f7efdbb6398d545446e246eff02"
uuid = "864edb3b-99cc-5e75-8d2d-829cb0a9cfe8"
version = "0.18.10"

[[DataValueInterfaces]]
git-tree-sha1 = "bfc1187b79289637fa0ef6d4436ebdfe6905cbd6"
uuid = "e2d170a0-9d28-54be-80f0-106bbe20a464"
version = "1.0.0"

[[Dates]]
deps = ["Printf"]
uuid = "ade2ca70-3891-5945-98fb-dc099432e06a"

[[DelimitedFiles]]
deps = ["Mmap"]
uuid = "8bb1440f-4735-579b-a4ab-409b98df4dab"

[[Distributed]]
deps = ["Random", "Serialization", "Sockets"]
uuid = "8ba89e20-285c-5b6f-9357-94700520ee1b"

[[Distributions]]
deps = ["ChainRulesCore", "FillArrays", "LinearAlgebra", "PDMats", "Printf", "QuadGK", "Random", "SparseArrays", "SpecialFunctions", "Statistics", "StatsBase", "StatsFuns"]
git-tree-sha1 = "ff7890c74e2eaffbc0b3741811e3816e64b6343d"
uuid = "31c24e10-a181-5473-b8eb-7969acd0382f"
version = "0.25.18"

[[DocStringExtensions]]
deps = ["LibGit2"]
git-tree-sha1 = "a32185f5428d3986f47c2ab78b1f216d5e6cc96f"
uuid = "ffbed154-4ef7-542d-bbb7-c09d3a79fcae"
version = "0.8.5"

[[Downloads]]
deps = ["ArgTools", "LibCURL", "NetworkOptions"]
git-tree-sha1 = "135bf1896be424235eadb17474b2a78331567f08"
uuid = "f43a241f-c20a-4ad4-852c-f6b1247861c6"
version = "1.5.1"

[[EarCut_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "92d8f9f208637e8d2d28c664051a00569c01493d"
uuid = "5ae413db-bbd1-5e63-b57d-d24a61df00f5"
version = "2.1.5+1"

[[Expat_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "1402e52fcda25064f51c77a9655ce8680b76acf0"
uuid = "2e619515-83b5-522b-bb60-26c02a35a201"
version = "2.2.7+6"

[[FFMPEG]]
deps = ["FFMPEG_jll"]
git-tree-sha1 = "b57e3acbe22f8484b4b5ff66a7499717fe1a9cc8"
uuid = "c87230d0-a227-11e9-1b43-d7ebe4e7570a"
version = "0.4.1"

[[FFMPEG_jll]]
deps = ["Artifacts", "Bzip2_jll", "FreeType2_jll", "FriBidi_jll", "JLLWrappers", "LAME_jll", "LibVPX_jll", "Libdl", "Ogg_jll", "OpenSSL_jll", "Opus_jll", "Pkg", "Zlib_jll", "libass_jll", "libfdk_aac_jll", "libvorbis_jll", "x264_jll", "x265_jll"]
git-tree-sha1 = "3cc57ad0a213808473eafef4845a74766242e05f"
uuid = "b22a6f82-2f65-5046-a5b2-351ab43fb4e5"
version = "4.3.1+4"

[[FillArrays]]
deps = ["LinearAlgebra", "Random", "SparseArrays", "Statistics"]
git-tree-sha1 = "29890dfbc427afa59598b8cfcc10034719bd7744"
uuid = "1a297f60-69ca-5386-bcde-b61e274b549b"
version = "0.12.6"

[[FixedPointNumbers]]
deps = ["Statistics"]
git-tree-sha1 = "335bfdceacc84c5cdf16aadc768aa5ddfc5383cc"
uuid = "53c48c17-4a7d-5ca2-90c5-79b7896eea93"
version = "0.8.4"

[[Fontconfig_jll]]
deps = ["Artifacts", "Bzip2_jll", "Expat_jll", "FreeType2_jll", "JLLWrappers", "Libdl", "Libuuid_jll", "Pkg", "Zlib_jll"]
git-tree-sha1 = "35895cf184ceaab11fd778b4590144034a167a2f"
uuid = "a3f928ae-7b40-5064-980b-68af3947d34b"
version = "2.13.1+14"

[[Formatting]]
deps = ["Printf"]
git-tree-sha1 = "8339d61043228fdd3eb658d86c926cb282ae72a8"
uuid = "59287772-0a20-5a39-b81b-1366585eb4c0"
version = "0.4.2"

[[FreeType2_jll]]
deps = ["Artifacts", "Bzip2_jll", "JLLWrappers", "Libdl", "Pkg", "Zlib_jll"]
git-tree-sha1 = "cbd58c9deb1d304f5a245a0b7eb841a2560cfec6"
uuid = "d7e528f0-a631-5988-bf34-fe36492bcfd7"
version = "2.10.1+5"

[[FriBidi_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "0d20aed5b14dd4c9a2453c1b601d08e1149679cc"
uuid = "559328eb-81f9-559d-9380-de523a88c83c"
version = "1.0.5+6"

[[GLFW_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Libglvnd_jll", "Pkg", "Xorg_libXcursor_jll", "Xorg_libXi_jll", "Xorg_libXinerama_jll", "Xorg_libXrandr_jll"]
git-tree-sha1 = "a199aefead29c3c2638c3571a9993b564109d45a"
uuid = "0656b61e-2033-5cc2-a64a-77c0f6c09b89"
version = "3.3.4+0"

[[GR]]
deps = ["Base64", "DelimitedFiles", "GR_jll", "HTTP", "JSON", "Libdl", "LinearAlgebra", "Pkg", "Printf", "Random", "Serialization", "Sockets", "Test", "UUIDs"]
git-tree-sha1 = "c2178cfbc0a5a552e16d097fae508f2024de61a3"
uuid = "28b8d3ca-fb5f-59d9-8090-bfdbd6d07a71"
version = "0.59.0"

[[GR_jll]]
deps = ["Artifacts", "Bzip2_jll", "Cairo_jll", "FFMPEG_jll", "Fontconfig_jll", "GLFW_jll", "JLLWrappers", "JpegTurbo_jll", "Libdl", "Libtiff_jll", "Pixman_jll", "Pkg", "Qt5Base_jll", "Zlib_jll", "libpng_jll"]
git-tree-sha1 = "d59e8320c2747553788e4fc42231489cc602fa50"
uuid = "d2c73de3-f751-5644-a686-071e5b155ba9"
version = "0.58.1+0"

[[GeometryBasics]]
deps = ["EarCut_jll", "IterTools", "LinearAlgebra", "StaticArrays", "StructArrays", "Tables"]
git-tree-sha1 = "58bcdf5ebc057b085e58d95c138725628dd7453c"
uuid = "5c1252a2-5f33-56bf-86c9-59e7332b4326"
version = "0.4.1"

[[Gettext_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Libiconv_jll", "Pkg", "XML2_jll"]
git-tree-sha1 = "8c14294a079216000a0bdca5ec5a447f073ddc9d"
uuid = "78b55507-aeef-58d4-861c-77aaff3498b1"
version = "0.20.1+7"

[[Glib_jll]]
deps = ["Artifacts", "Gettext_jll", "JLLWrappers", "Libdl", "Libffi_jll", "Libiconv_jll", "Libmount_jll", "PCRE_jll", "Pkg", "Zlib_jll"]
git-tree-sha1 = "04690cc5008b38ecbdfede949220bc7d9ba26397"
uuid = "7746bdde-850d-59dc-9ae8-88ece973131d"
version = "2.59.0+4"

[[Grisu]]
git-tree-sha1 = "53bb909d1151e57e2484c3d1b53e19552b887fb2"
uuid = "42e2da0e-8278-4e71-bc24-59509adca0fe"
version = "1.0.2"

[[HTTP]]
deps = ["Base64", "Dates", "IniFile", "Logging", "MbedTLS", "NetworkOptions", "Sockets", "URIs"]
git-tree-sha1 = "14eece7a3308b4d8be910e265c724a6ba51a9798"
uuid = "cd3eb016-35fb-5094-929b-558a96fad6f3"
version = "0.9.16"

[[Hyperscript]]
deps = ["Test"]
git-tree-sha1 = "8d511d5b81240fc8e6802386302675bdf47737b9"
uuid = "47d2ed2b-36de-50cf-bf87-49c2cf4b8b91"
version = "0.0.4"

[[HypertextLiteral]]
git-tree-sha1 = "f6532909bf3d40b308a0f360b6a0e626c0e263a8"
uuid = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
version = "0.9.1"

[[IOCapture]]
deps = ["Logging", "Random"]
git-tree-sha1 = "f7be53659ab06ddc986428d3a9dcc95f6fa6705a"
uuid = "b5f81e59-6552-4d32-b1f0-c071b021bf89"
version = "0.2.2"

[[IniFile]]
deps = ["Test"]
git-tree-sha1 = "098e4d2c533924c921f9f9847274f2ad89e018b8"
uuid = "83e8ac13-25f8-5344-8a64-a9f2b223428f"
version = "0.5.0"

[[InteractiveUtils]]
deps = ["Markdown"]
uuid = "b77e0a4c-d291-57a0-90e8-8db25a27a240"

[[IrrationalConstants]]
git-tree-sha1 = "f76424439413893a832026ca355fe273e93bce94"
uuid = "92d709cd-6900-40b7-9082-c6be49f344b6"
version = "0.1.0"

[[IterTools]]
git-tree-sha1 = "05110a2ab1fc5f932622ffea2a003221f4782c18"
uuid = "c8e1da08-722c-5040-9ed9-7db0dc04731e"
version = "1.3.0"

[[IteratorInterfaceExtensions]]
git-tree-sha1 = "a3f24677c21f5bbe9d2a714f95dcd58337fb2856"
uuid = "82899510-4779-5014-852e-03e436cf321d"
version = "1.0.0"

[[JLLWrappers]]
deps = ["Preferences"]
git-tree-sha1 = "642a199af8b68253517b80bd3bfd17eb4e84df6e"
uuid = "692b3bcd-3c85-4b1f-b108-f13ce0eb3210"
version = "1.3.0"

[[JSON]]
deps = ["Dates", "Mmap", "Parsers", "Unicode"]
git-tree-sha1 = "8076680b162ada2a031f707ac7b4953e30667a37"
uuid = "682c06a0-de6a-54ab-a142-c8b1cf79cde6"
version = "0.21.2"

[[JpegTurbo_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "9aff0587d9603ea0de2c6f6300d9f9492bbefbd3"
uuid = "aacddb02-875f-59d6-b918-886e6ef4fbf8"
version = "2.0.1+3"

[[LAME_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "df381151e871f41ee86cee4f5f6fd598b8a68826"
uuid = "c1c5ebd0-6772-5130-a774-d5fcae4a789d"
version = "3.100.0+3"

[[LZO_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "f128cd6cd05ffd6d3df0523ed99b90ff6f9b349a"
uuid = "dd4b983a-f0e5-5f8d-a1b7-129d4a5fb1ac"
version = "2.10.0+3"

[[LaTeXStrings]]
git-tree-sha1 = "c7f1c695e06c01b95a67f0cd1d34994f3e7db104"
uuid = "b964fa9f-0449-5b57-a5c2-d3ea65f4040f"
version = "1.2.1"

[[Latexify]]
deps = ["Formatting", "InteractiveUtils", "LaTeXStrings", "MacroTools", "Markdown", "Printf", "Requires"]
git-tree-sha1 = "a4b12a1bd2ebade87891ab7e36fdbce582301a92"
uuid = "23fbe1c1-3f47-55db-b15f-69d7ec21a316"
version = "0.15.6"

[[LibCURL]]
deps = ["LibCURL_jll", "MozillaCACerts_jll"]
git-tree-sha1 = "cdbe7465ab7b52358804713a53c7fe1dac3f8a3f"
uuid = "b27032c2-a3e7-50c8-80cd-2d36dbcbfd21"
version = "0.6.3"

[[LibCURL_jll]]
deps = ["LibSSH2_jll", "Libdl", "MbedTLS_jll", "Pkg", "Zlib_jll", "nghttp2_jll"]
git-tree-sha1 = "897d962c20031e6012bba7b3dcb7a667170dad17"
uuid = "deac9b47-8bc7-5906-a0fe-35ac56dc84c0"
version = "7.70.0+2"

[[LibGit2]]
deps = ["Printf"]
uuid = "76f85450-5226-5b5a-8eaa-529ad045b433"

[[LibSSH2_jll]]
deps = ["Libdl", "MbedTLS_jll", "Pkg"]
git-tree-sha1 = "717705533148132e5466f2924b9a3657b16158e8"
uuid = "29816b5a-b9ab-546f-933c-edad1886dfa8"
version = "1.9.0+3"

[[LibVPX_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "85fcc80c3052be96619affa2fe2e6d2da3908e11"
uuid = "dd192d2f-8180-539f-9fb4-cc70b1dcf69a"
version = "1.9.0+1"

[[Libdl]]
uuid = "8f399da3-3557-5675-b5ff-fb832c97cbdb"

[[Libffi_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "a2cd088a88c0d37eef7d209fd3d8712febce0d90"
uuid = "e9f186c6-92d2-5b65-8a66-fee21dc1b490"
version = "3.2.1+4"

[[Libgcrypt_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Libgpg_error_jll", "Pkg"]
git-tree-sha1 = "b391a18ab1170a2e568f9fb8d83bc7c780cb9999"
uuid = "d4300ac3-e22c-5743-9152-c294e39db1e4"
version = "1.8.5+4"

[[Libglvnd_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libX11_jll", "Xorg_libXext_jll"]
git-tree-sha1 = "7739f837d6447403596a75d19ed01fd08d6f56bf"
uuid = "7e76a0d4-f3c7-5321-8279-8d96eeed0f29"
version = "1.3.0+3"

[[Libgpg_error_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "ec7f2e8ad5c9fa99fc773376cdbc86d9a5a23cb7"
uuid = "7add5ba3-2f88-524e-9cd5-f83b8a55f7b8"
version = "1.36.0+3"

[[Libiconv_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "cba7b560fcc00f8cd770fa85a498cbc1d63ff618"
uuid = "94ce4f54-9a6c-5748-9c1c-f9c7231a4531"
version = "1.16.0+8"

[[Libmount_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "51ad0c01c94c1ce48d5cad629425035ad030bfd5"
uuid = "4b2f31a3-9ecc-558c-b454-b3730dcb73e9"
version = "2.34.0+3"

[[Libtiff_jll]]
deps = ["Artifacts", "JLLWrappers", "JpegTurbo_jll", "Libdl", "Pkg", "Zlib_jll", "Zstd_jll"]
git-tree-sha1 = "291dd857901f94d683973cdf679984cdf73b56d0"
uuid = "89763e89-9b03-5906-acba-b20f662cd828"
version = "4.1.0+2"

[[Libuuid_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "f879ae9edbaa2c74c922e8b85bb83cc84ea1450b"
uuid = "38a345b3-de98-5d2b-a5d3-14cd9215e700"
version = "2.34.0+7"

[[LinearAlgebra]]
deps = ["Libdl"]
uuid = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"

[[LogExpFunctions]]
deps = ["ChainRulesCore", "DocStringExtensions", "IrrationalConstants", "LinearAlgebra"]
git-tree-sha1 = "34dc30f868e368f8a17b728a1238f3fcda43931a"
uuid = "2ab3a3ac-af41-5b50-aa03-7779005ae688"
version = "0.3.3"

[[Logging]]
uuid = "56ddb016-857b-54e1-b83d-db4d58db5568"

[[MacroTools]]
deps = ["Markdown", "Random"]
git-tree-sha1 = "5a5bc6bf062f0f95e62d0fe0a2d99699fed82dd9"
uuid = "1914dd2f-81c6-5fcd-8719-6d5c9610ff09"
version = "0.5.8"

[[Markdown]]
deps = ["Base64"]
uuid = "d6f4376e-aef5-505a-96c1-9c027394607a"

[[MbedTLS]]
deps = ["Dates", "MbedTLS_jll", "Random", "Sockets"]
git-tree-sha1 = "1c38e51c3d08ef2278062ebceade0e46cefc96fe"
uuid = "739be429-bea8-5141-9913-cc70e7f3736d"
version = "1.0.3"

[[MbedTLS_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "0eef589dd1c26a3ac9d753fe1a8bcad63f956fa6"
uuid = "c8ffd9c3-330d-5841-b78e-0817d7145fa1"
version = "2.16.8+1"

[[Measures]]
git-tree-sha1 = "e498ddeee6f9fdb4551ce855a46f54dbd900245f"
uuid = "442fdcdd-2543-5da2-b0f3-8c86c306513e"
version = "0.3.1"

[[Missings]]
deps = ["DataAPI"]
git-tree-sha1 = "bf210ce90b6c9eed32d25dbcae1ebc565df2687f"
uuid = "e1d29d7a-bbdc-5cf2-9ac0-f12de2c33e28"
version = "1.0.2"

[[Mmap]]
uuid = "a63ad114-7e13-5084-954f-fe012c677804"

[[MozillaCACerts_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "f1662575f7bf53c73c2bbc763bace4b024de822c"
uuid = "14a3606d-f60d-562e-9121-12d972cd8159"
version = "2021.1.19+0"

[[NaNMath]]
git-tree-sha1 = "bfe47e760d60b82b66b61d2d44128b62e3a369fb"
uuid = "77ba4419-2d1f-58cd-9bb1-8ffee604a2e3"
version = "0.3.5"

[[NetworkOptions]]
git-tree-sha1 = "ed3157f48a05543cce9b241e1f2815f7e843d96e"
uuid = "ca575930-c2e3-43a9-ace4-1e988b2c1908"
version = "1.2.0"

[[Ogg_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "a42c0f138b9ebe8b58eba2271c5053773bde52d0"
uuid = "e7412a2a-1a6e-54c0-be00-318e2571c051"
version = "1.3.4+2"

[[OpenLibm_jll]]
deps = ["Libdl", "Pkg"]
git-tree-sha1 = "d22054f66695fe580009c09e765175cbf7f13031"
uuid = "05823500-19ac-5b8b-9628-191a04bc5112"
version = "0.7.1+0"

[[OpenSSL_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "71bbbc616a1d710879f5a1021bcba65ffba6ce58"
uuid = "458c3c95-2e84-50aa-8efc-19380b2a3a95"
version = "1.1.1+6"

[[OpenSpecFun_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "9db77584158d0ab52307f8c04f8e7c08ca76b5b3"
uuid = "efe28fd5-8261-553b-a9e1-b2916fc3738e"
version = "0.5.3+4"

[[Opus_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "f9d57f4126c39565e05a2b0264df99f497fc6f37"
uuid = "91d4177d-7536-5919-b921-800302f37372"
version = "1.3.1+3"

[[OrderedCollections]]
git-tree-sha1 = "85f8e6578bf1f9ee0d11e7bb1b1456435479d47c"
uuid = "bac558e1-5e72-5ebc-8fee-abe8a469f55d"
version = "1.4.1"

[[PCRE_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "1b556ad51dceefdbf30e86ffa8f528b73c7df2bb"
uuid = "2f80f16e-611a-54ab-bc61-aa92de5b98fc"
version = "8.42.0+4"

[[PDMats]]
deps = ["LinearAlgebra", "SparseArrays", "SuiteSparse"]
git-tree-sha1 = "4dd403333bcf0909341cfe57ec115152f937d7d8"
uuid = "90014a1f-27ba-587c-ab20-58faa44d9150"
version = "0.11.1"

[[Parsers]]
deps = ["Dates"]
git-tree-sha1 = "a8709b968a1ea6abc2dc1967cb1db6ac9a00dfb6"
uuid = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"
version = "2.0.5"

[[Pixman_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "6a20a83c1ae86416f0a5de605eaea08a552844a3"
uuid = "30392449-352a-5448-841d-b1acce4e97dc"
version = "0.40.0+0"

[[Pkg]]
deps = ["Dates", "LibGit2", "Libdl", "Logging", "Markdown", "Printf", "REPL", "Random", "SHA", "UUIDs"]
uuid = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"

[[PlotThemes]]
deps = ["PlotUtils", "Requires", "Statistics"]
git-tree-sha1 = "a3a964ce9dc7898193536002a6dd892b1b5a6f1d"
uuid = "ccf2f8ad-2431-5c83-bf29-c5338b663b6a"
version = "2.0.1"

[[PlotUtils]]
deps = ["ColorSchemes", "Colors", "Dates", "Printf", "Random", "Reexport", "Statistics"]
git-tree-sha1 = "b084324b4af5a438cd63619fd006614b3b20b87b"
uuid = "995b91a9-d308-5afd-9ec6-746e21dbc043"
version = "1.0.15"

[[Plots]]
deps = ["Base64", "Contour", "Dates", "Downloads", "FFMPEG", "FixedPointNumbers", "GR", "GeometryBasics", "JSON", "Latexify", "LinearAlgebra", "Measures", "NaNMath", "PlotThemes", "PlotUtils", "Printf", "REPL", "Random", "RecipesBase", "RecipesPipeline", "Reexport", "Requires", "Scratch", "Showoff", "SparseArrays", "Statistics", "StatsBase", "UUIDs"]
git-tree-sha1 = "6841db754bd01a91d281370d9a0f8787e220ae08"
uuid = "91a5bcdd-55d7-5caf-9e0b-520d859cae80"
version = "1.22.4"

[[PlutoUI]]
deps = ["Base64", "Dates", "Hyperscript", "HypertextLiteral", "IOCapture", "InteractiveUtils", "JSON", "Logging", "Markdown", "Random", "Reexport", "UUIDs"]
git-tree-sha1 = "4c8a7d080daca18545c56f1cac28710c362478f3"
uuid = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
version = "0.7.16"

[[Preferences]]
deps = ["TOML"]
git-tree-sha1 = "00cfd92944ca9c760982747e9a1d0d5d86ab1e5a"
uuid = "21216c6a-2e73-6563-6e65-726566657250"
version = "1.2.2"

[[Printf]]
deps = ["Unicode"]
uuid = "de0858da-6303-5e67-8744-51eddeeeb8d7"

[[Qt5Base_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "Fontconfig_jll", "Glib_jll", "JLLWrappers", "Libdl", "Libglvnd_jll", "OpenSSL_jll", "Pkg", "Xorg_libXext_jll", "Xorg_libxcb_jll", "Xorg_xcb_util_image_jll", "Xorg_xcb_util_keysyms_jll", "Xorg_xcb_util_renderutil_jll", "Xorg_xcb_util_wm_jll", "Zlib_jll", "xkbcommon_jll"]
git-tree-sha1 = "16626cfabbf7206d60d84f2bf4725af7b37d4a77"
uuid = "ea2cea3b-5b76-57ae-a6ef-0a8af62496e1"
version = "5.15.2+0"

[[QuadGK]]
deps = ["DataStructures", "LinearAlgebra"]
git-tree-sha1 = "78aadffb3efd2155af139781b8a8df1ef279ea39"
uuid = "1fd47b50-473d-5c70-9696-f719f8f3bcdc"
version = "2.4.2"

[[REPL]]
deps = ["InteractiveUtils", "Markdown", "Sockets"]
uuid = "3fa0cd96-eef1-5676-8a61-b3b8758bbffb"

[[Random]]
deps = ["Serialization"]
uuid = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"

[[RecipesBase]]
git-tree-sha1 = "44a75aa7a527910ee3d1751d1f0e4148698add9e"
uuid = "3cdcf5f2-1ef4-517c-9805-6587b60abb01"
version = "1.1.2"

[[RecipesPipeline]]
deps = ["Dates", "NaNMath", "PlotUtils", "RecipesBase"]
git-tree-sha1 = "7ad0dfa8d03b7bcf8c597f59f5292801730c55b8"
uuid = "01d81517-befc-4cb6-b9ec-a95719d0359c"
version = "0.4.1"

[[Reexport]]
git-tree-sha1 = "45e428421666073eab6f2da5c9d310d99bb12f9b"
uuid = "189a3867-3050-52da-a836-e630ba90ab69"
version = "1.2.2"

[[Requires]]
deps = ["UUIDs"]
git-tree-sha1 = "4036a3bd08ac7e968e27c203d45f5fff15020621"
uuid = "ae029012-a4dd-5104-9daa-d747884805df"
version = "1.1.3"

[[Rmath]]
deps = ["Random", "Rmath_jll"]
git-tree-sha1 = "86c5647b565873641538d8f812c04e4c9dbeb370"
uuid = "79098fc4-a85e-5d69-aa6a-4863f24498fa"
version = "0.6.1"

[[Rmath_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "1b7bf41258f6c5c9c31df8c1ba34c1fc88674957"
uuid = "f50d1b31-88e8-58de-be2c-1cc44531875f"
version = "0.2.2+2"

[[SHA]]
uuid = "ea8e919c-243c-51af-8825-aaa63cd721ce"

[[Scratch]]
deps = ["Dates"]
git-tree-sha1 = "0b4b7f1393cff97c33891da2a0bf69c6ed241fda"
uuid = "6c6a2e73-6563-6170-7368-637461726353"
version = "1.1.0"

[[Serialization]]
uuid = "9e88b42a-f829-5b0c-bbe9-9e923198166b"

[[SharedArrays]]
deps = ["Distributed", "Mmap", "Random", "Serialization"]
uuid = "1a1011a3-84de-559e-8e89-a11a2f7dc383"

[[Showoff]]
deps = ["Dates", "Grisu"]
git-tree-sha1 = "91eddf657aca81df9ae6ceb20b959ae5653ad1de"
uuid = "992d4aef-0814-514b-bc4d-f2e9a6c4116f"
version = "1.0.3"

[[Sockets]]
uuid = "6462fe0b-24de-5631-8697-dd941f90decc"

[[SortingAlgorithms]]
deps = ["DataStructures"]
git-tree-sha1 = "b3363d7460f7d098ca0912c69b082f75625d7508"
uuid = "a2af1166-a08f-5f64-846c-94a0d3cef48c"
version = "1.0.1"

[[SparseArrays]]
deps = ["LinearAlgebra", "Random"]
uuid = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"

[[SpecialFunctions]]
deps = ["ChainRulesCore", "IrrationalConstants", "LogExpFunctions", "OpenLibm_jll", "OpenSpecFun_jll"]
git-tree-sha1 = "793793f1df98e3d7d554b65a107e9c9a6399a6ed"
uuid = "276daf66-3868-5448-9aa4-cd146d93841b"
version = "1.7.0"

[[StaticArrays]]
deps = ["LinearAlgebra", "Random", "Statistics"]
git-tree-sha1 = "3c76dde64d03699e074ac02eb2e8ba8254d428da"
uuid = "90137ffa-7385-5640-81b9-e52037218182"
version = "1.2.13"

[[Statistics]]
deps = ["LinearAlgebra", "SparseArrays"]
uuid = "10745b16-79ce-11e8-11f9-7d13ad32a3b2"

[[StatsAPI]]
git-tree-sha1 = "1958272568dc176a1d881acb797beb909c785510"
uuid = "82ae8749-77ed-4fe6-ae5f-f523153014b0"
version = "1.0.0"

[[StatsBase]]
deps = ["DataAPI", "DataStructures", "LinearAlgebra", "Missings", "Printf", "Random", "SortingAlgorithms", "SparseArrays", "Statistics", "StatsAPI"]
git-tree-sha1 = "8cbbc098554648c84f79a463c9ff0fd277144b6c"
uuid = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"
version = "0.33.10"

[[StatsFuns]]
deps = ["ChainRulesCore", "IrrationalConstants", "LogExpFunctions", "Reexport", "Rmath", "SpecialFunctions"]
git-tree-sha1 = "95072ef1a22b057b1e80f73c2a89ad238ae4cfff"
uuid = "4c63d2b9-4356-54db-8cca-17b64c39e42c"
version = "0.9.12"

[[StructArrays]]
deps = ["Adapt", "DataAPI", "StaticArrays", "Tables"]
git-tree-sha1 = "2ce41e0d042c60ecd131e9fb7154a3bfadbf50d3"
uuid = "09ab397b-f2b6-538f-b94a-2f83cf4a842a"
version = "0.6.3"

[[SuiteSparse]]
deps = ["Libdl", "LinearAlgebra", "Serialization", "SparseArrays"]
uuid = "4607b0f0-06f3-5cda-b6b1-a6196a1729e9"

[[TOML]]
deps = ["Dates"]
git-tree-sha1 = "44aaac2d2aec4a850302f9aa69127c74f0c3787e"
uuid = "fa267f1f-6049-4f14-aa54-33bafae1ed76"
version = "1.0.3"

[[TableTraits]]
deps = ["IteratorInterfaceExtensions"]
git-tree-sha1 = "c06b2f539df1c6efa794486abfb6ed2022561a39"
uuid = "3783bdb8-4a98-5b6b-af9a-565f29a5fe9c"
version = "1.0.1"

[[Tables]]
deps = ["DataAPI", "DataValueInterfaces", "IteratorInterfaceExtensions", "LinearAlgebra", "TableTraits", "Test"]
git-tree-sha1 = "fed34d0e71b91734bf0a7e10eb1bb05296ddbcd0"
uuid = "bd369af6-aec1-5ad0-b16a-f7cc5008161c"
version = "1.6.0"

[[Test]]
deps = ["Distributed", "InteractiveUtils", "Logging", "Random"]
uuid = "8dfed614-e22c-5e08-85e1-65c5234f0b40"

[[URIs]]
git-tree-sha1 = "97bbe755a53fe859669cd907f2d96aee8d2c1355"
uuid = "5c2747f8-b7ea-4ff2-ba2e-563bfd36b1d4"
version = "1.3.0"

[[UUIDs]]
deps = ["Random", "SHA"]
uuid = "cf7118a7-6976-5b1a-9a39-7adc72f591a4"

[[Unicode]]
uuid = "4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5"

[[Wayland_jll]]
deps = ["Artifacts", "Expat_jll", "JLLWrappers", "Libdl", "Libffi_jll", "Pkg", "XML2_jll"]
git-tree-sha1 = "dc643a9b774da1c2781413fd7b6dcd2c56bb8056"
uuid = "a2964d1f-97da-50d4-b82a-358c7fce9d89"
version = "1.17.0+4"

[[Wayland_protocols_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Wayland_jll"]
git-tree-sha1 = "2839f1c1296940218e35df0bbb220f2a79686670"
uuid = "2381bf8a-dfd0-557d-9999-79630e7b1b91"
version = "1.18.0+4"

[[XML2_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Libiconv_jll", "Pkg", "Zlib_jll"]
git-tree-sha1 = "be0db24f70aae7e2b89f2f3092e93b8606d659a6"
uuid = "02c8fc9c-b97f-50b9-bbe4-9be30ff0a78a"
version = "2.9.10+3"

[[XSLT_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Libgcrypt_jll", "Libgpg_error_jll", "Pkg", "XML2_jll"]
git-tree-sha1 = "2b3eac39df218762d2d005702d601cd44c997497"
uuid = "aed1982a-8fda-507f-9586-7b0439959a61"
version = "1.1.33+4"

[[Xorg_libX11_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libxcb_jll", "Xorg_xtrans_jll"]
git-tree-sha1 = "5be649d550f3f4b95308bf0183b82e2582876527"
uuid = "4f6342f7-b3d2-589e-9d20-edeb45f2b2bc"
version = "1.6.9+4"

[[Xorg_libXau_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "4e490d5c960c314f33885790ed410ff3a94ce67e"
uuid = "0c0b7dd1-d40b-584c-a123-a41640f87eec"
version = "1.0.9+4"

[[Xorg_libXcursor_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libXfixes_jll", "Xorg_libXrender_jll"]
git-tree-sha1 = "12e0eb3bc634fa2080c1c37fccf56f7c22989afd"
uuid = "935fb764-8cf2-53bf-bb30-45bb1f8bf724"
version = "1.2.0+4"

[[Xorg_libXdmcp_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "4fe47bd2247248125c428978740e18a681372dd4"
uuid = "a3789734-cfe1-5b06-b2d0-1dd0d9d62d05"
version = "1.1.3+4"

[[Xorg_libXext_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libX11_jll"]
git-tree-sha1 = "b7c0aa8c376b31e4852b360222848637f481f8c3"
uuid = "1082639a-0dae-5f34-9b06-72781eeb8cb3"
version = "1.3.4+4"

[[Xorg_libXfixes_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libX11_jll"]
git-tree-sha1 = "0e0dc7431e7a0587559f9294aeec269471c991a4"
uuid = "d091e8ba-531a-589c-9de9-94069b037ed8"
version = "5.0.3+4"

[[Xorg_libXi_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libXext_jll", "Xorg_libXfixes_jll"]
git-tree-sha1 = "89b52bc2160aadc84d707093930ef0bffa641246"
uuid = "a51aa0fd-4e3c-5386-b890-e753decda492"
version = "1.7.10+4"

[[Xorg_libXinerama_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libXext_jll"]
git-tree-sha1 = "26be8b1c342929259317d8b9f7b53bf2bb73b123"
uuid = "d1454406-59df-5ea1-beac-c340f2130bc3"
version = "1.1.4+4"

[[Xorg_libXrandr_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libXext_jll", "Xorg_libXrender_jll"]
git-tree-sha1 = "34cea83cb726fb58f325887bf0612c6b3fb17631"
uuid = "ec84b674-ba8e-5d96-8ba1-2a689ba10484"
version = "1.5.2+4"

[[Xorg_libXrender_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libX11_jll"]
git-tree-sha1 = "19560f30fd49f4d4efbe7002a1037f8c43d43b96"
uuid = "ea2f1a96-1ddc-540d-b46f-429655e07cfa"
version = "0.9.10+4"

[[Xorg_libpthread_stubs_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "6783737e45d3c59a4a4c4091f5f88cdcf0908cbb"
uuid = "14d82f49-176c-5ed1-bb49-ad3f5cbd8c74"
version = "0.1.0+3"

[[Xorg_libxcb_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "XSLT_jll", "Xorg_libXau_jll", "Xorg_libXdmcp_jll", "Xorg_libpthread_stubs_jll"]
git-tree-sha1 = "daf17f441228e7a3833846cd048892861cff16d6"
uuid = "c7cfdc94-dc32-55de-ac96-5a1b8d977c5b"
version = "1.13.0+3"

[[Xorg_libxkbfile_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libX11_jll"]
git-tree-sha1 = "926af861744212db0eb001d9e40b5d16292080b2"
uuid = "cc61e674-0454-545c-8b26-ed2c68acab7a"
version = "1.1.0+4"

[[Xorg_xcb_util_image_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_xcb_util_jll"]
git-tree-sha1 = "0fab0a40349ba1cba2c1da699243396ff8e94b97"
uuid = "12413925-8142-5f55-bb0e-6d7ca50bb09b"
version = "0.4.0+1"

[[Xorg_xcb_util_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libxcb_jll"]
git-tree-sha1 = "e7fd7b2881fa2eaa72717420894d3938177862d1"
uuid = "2def613f-5ad1-5310-b15b-b15d46f528f5"
version = "0.4.0+1"

[[Xorg_xcb_util_keysyms_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_xcb_util_jll"]
git-tree-sha1 = "d1151e2c45a544f32441a567d1690e701ec89b00"
uuid = "975044d2-76e6-5fbe-bf08-97ce7c6574c7"
version = "0.4.0+1"

[[Xorg_xcb_util_renderutil_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_xcb_util_jll"]
git-tree-sha1 = "dfd7a8f38d4613b6a575253b3174dd991ca6183e"
uuid = "0d47668e-0667-5a69-a72c-f761630bfb7e"
version = "0.3.9+1"

[[Xorg_xcb_util_wm_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_xcb_util_jll"]
git-tree-sha1 = "e78d10aab01a4a154142c5006ed44fd9e8e31b67"
uuid = "c22f9ab0-d5fe-5066-847c-f4bb1cd4e361"
version = "0.4.1+1"

[[Xorg_xkbcomp_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libxkbfile_jll"]
git-tree-sha1 = "4bcbf660f6c2e714f87e960a171b119d06ee163b"
uuid = "35661453-b289-5fab-8a00-3d9160c6a3a4"
version = "1.4.2+4"

[[Xorg_xkeyboard_config_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_xkbcomp_jll"]
git-tree-sha1 = "5c8424f8a67c3f2209646d4425f3d415fee5931d"
uuid = "33bec58e-1273-512f-9401-5d533626f822"
version = "2.27.0+4"

[[Xorg_xtrans_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "79c31e7844f6ecf779705fbc12146eb190b7d845"
uuid = "c5fb5394-a638-5e4d-96e5-b29de1b5cf10"
version = "1.4.0+3"

[[Zlib_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "320228915c8debb12cb434c59057290f0834dbf6"
uuid = "83775a58-1f1d-513f-b197-d71354ab007a"
version = "1.2.11+18"

[[Zstd_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "2c1332c54931e83f8f94d310fa447fd743e8d600"
uuid = "3161d3a3-bdf6-5164-811a-617609db77b4"
version = "1.4.8+0"

[[libass_jll]]
deps = ["Artifacts", "Bzip2_jll", "FreeType2_jll", "FriBidi_jll", "JLLWrappers", "Libdl", "Pkg", "Zlib_jll"]
git-tree-sha1 = "acc685bcf777b2202a904cdcb49ad34c2fa1880c"
uuid = "0ac62f75-1d6f-5e53-bd7c-93b484bb37c0"
version = "0.14.0+4"

[[libfdk_aac_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "7a5780a0d9c6864184b3a2eeeb833a0c871f00ab"
uuid = "f638f0a6-7fb0-5443-88ba-1cc74229b280"
version = "0.1.6+4"

[[libpng_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Zlib_jll"]
git-tree-sha1 = "6abbc424248097d69c0c87ba50fcb0753f93e0ee"
uuid = "b53b4c65-9356-5827-b1ea-8c7a1a84506f"
version = "1.6.37+6"

[[libvorbis_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Ogg_jll", "Pkg"]
git-tree-sha1 = "fa14ac25af7a4b8a7f61b287a124df7aab601bcd"
uuid = "f27f6e37-5d2b-51aa-960f-b287f2bc3b7a"
version = "1.3.6+6"

[[nghttp2_jll]]
deps = ["Libdl", "Pkg"]
git-tree-sha1 = "8e2c44ab4d49ad9518f359ed8b62f83ba8beede4"
uuid = "8e850ede-7688-5339-a07c-302acd2aaf8d"
version = "1.40.0+2"

[[x264_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "d713c1ce4deac133e3334ee12f4adff07f81778f"
uuid = "1270edf5-f2f9-52d2-97e9-ab00b5d0237a"
version = "2020.7.14+2"

[[x265_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "487da2f8f2f0c8ee0e83f39d13037d6bbf0a45ab"
uuid = "dfaa095f-4041-5dcd-9319-2fabd8486b76"
version = "3.0.0+3"

[[xkbcommon_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Wayland_jll", "Wayland_protocols_jll", "Xorg_libxcb_jll", "Xorg_xkeyboard_config_jll"]
git-tree-sha1 = "ece2350174195bb31de1a63bea3a41ae1aa593b6"
uuid = "d8fb68d0-12a3-5cfd-a85a-d49703b185fd"
version = "0.9.1+5"
"""

# ╔═╡ Cell order:
# ╟─41beadc2-385e-42bf-9960-ab201242b400
# ╠═ee082fcf-54d1-4aea-9944-8d7f81c6dbf4
# ╟─4d246460-af05-11eb-382b-590e60ba61f5
# ╟─ad7e5e06-55b2-4752-9335-2364489932eb
# ╟─d6b7e9f5-3bda-4477-b92c-e32b836f1f0d
# ╟─50462b1a-f65e-4d91-8d8d-da9c93ad007c
# ╟─f3ef2715-da53-449e-b198-faeeb78ac83f
# ╠═08054214-4045-486a-ac44-7a1a8aee9acf
# ╟─50d87809-df3b-4558-97ce-0a50fdfbcf69
# ╟─857da523-7c09-4230-9397-2dc0ef639007
# ╟─737c8375-2c49-4f32-b54c-1f15f8d451d5
# ╟─94bcc8de-a0be-47ab-a03a-b04c351ad6f0
# ╟─364c38ca-8637-4d26-8d64-525222d24033
# ╠═5841ea1a-8d23-4fe7-8010-7d8512050c22
# ╟─e1444e8d-c0ae-408f-ab88-fd11459d9842
# ╟─5c906c4a-8785-4199-8aba-a35c855e77f6
# ╟─449ad144-90d4-4f74-867f-939b489ddaad
# ╠═28dda5a1-546a-411f-9b8b-398e37195b13
# ╠═aad92f19-6935-4cbd-8e0f-d16a0d379386
# ╠═69fe641d-afb9-45dc-9aa0-436177b02ac3
# ╠═e88e908f-be65-488f-a0dd-718a091f9d5c
# ╠═480d6282-ce97-4127-957d-d0b9fd37a7e4
# ╟─55e0b804-a3ee-4667-8367-38b1b20b0096
# ╠═d021a50b-3bd1-4125-a939-1bba5eace898
# ╟─2bedff06-473e-49c0-9d8c-a0cde03ad554
# ╠═fa7b1373-006c-48e8-917b-c750bce86e09
# ╟─2ac080a7-af56-41ae-b913-8d65dedb9197
# ╟─b4cc09e0-2a6a-4ab6-8153-cbc817dede2e
# ╟─3466f636-7b06-42e0-b71e-55a7a2c74808
# ╠═0a99939a-4edb-488d-bbdf-4a425c808607
# ╠═f35928d5-e8c2-4c9f-b4ba-9cb58eafb9e0
# ╠═b8fe484e-1189-4fd0-85b9-715e084d65da
# ╟─d4cd5ff7-88b8-4e5c-a540-04c51ad51f42
# ╟─224d556c-cd9d-4c00-9cde-1c3df7cfcdef
# ╟─aa29947a-4dcb-48d8-9ad0-3828f8d4b8d2
# ╟─a27384c0-db19-4b06-8165-208d3fe77350
# ╟─7b43e546-07f0-45ce-9d2a-52894adad822
# ╟─ca7ecf8b-634e-4fa5-aaa8-102fa488c7a1
# ╟─60c3ce0d-85be-42f0-a869-1895abc096f3
# ╠═7eb18559-e2c0-4c34-a2b2-4e3c3ab831a6
# ╟─ecf6292c-7cdd-4886-8a59-be8e4f2751b7
# ╠═56f21296-fcd2-46f7-a130-4cb6940c1133
# ╠═09d77a14-01f4-430c-9038-25b85b31a1c2
# ╠═071cdabe-3443-490c-ab79-e270a0a1de68
# ╟─c59c5fb7-7291-46e2-88af-03ed38c7c3eb
# ╟─bb990e0c-df90-4635-80e4-e6639a62865a
# ╟─57309e16-8c1b-4268-b164-0b45b60b4fbd
# ╟─ca947d4b-18e6-4bad-8d9b-b73324a5c40b
# ╟─dc696281-7a5b-4568-a4c2-8dde90af43f0
# ╟─7968de5e-5ae8-4ab4-b089-c3d33475af2f
# ╟─be8e6332-f79d-4d63-afae-51c2d829f998
# ╟─f0eaf96b-0bc0-4194-9a36-886cb1d66e00
# ╟─293386f0-aafe-4e94-8e16-3237659d6963
# ╟─b2111d05-a153-4969-aeae-a7f0c01e3365
# ╟─f92a6b6e-a556-45cb-a1ae-9f5fe791ffd2
# ╠═667fc6a8-b3ff-464d-903c-e215e7d2f472
# ╠═ead64d36-947a-4b9f-a0f7-a1821039f5b3
# ╟─60fff6ab-3e19-4af7-b102-17d3d47494f3
# ╠═a041652b-365e-4594-9c48-c63d547b3295
# ╟─cc4c712d-6562-4d25-8b84-64458cda4198
# ╟─752392c9-198a-4d5a-8ce7-7c154ce834ec
# ╟─1f2c0903-13af-48d5-bbf5-b6102043a288
# ╟─a6014a30-6643-4504-8081-17bcc8be2615
# ╟─45fd4574-679f-42b8-9aa5-b949d252e2e7
# ╟─8032bbab-67a8-4058-873a-a33c4b39066f
# ╠═5f480d13-2dd1-451e-adf5-45d5d7749fdc
# ╟─da16c262-4d76-4757-9e80-df6871546278
# ╟─8b0ca97d-e7fd-4720-80a9-87c43c1f78a9
# ╟─a4e1a017-c37c-4a81-915c-96638964eda2
# ╠═12afa1a7-b4ed-4767-aff8-07dab6fa5d3c
# ╟─99d3250d-2335-425e-b470-2f570cba7367
# ╟─dad9f7f9-a5b0-44b2-b1a9-96b0fe9b3c7c
# ╟─16fd6703-d559-444c-8f42-26e314f29fb3
# ╠═63eed55a-1dfe-4b3d-b7aa-e4736718b105
# ╟─55631080-6d7d-4059-b8c1-32fa10ef8884
# ╟─efac7679-df64-4db2-8b8b-06febdc8a5f0
# ╠═d26ab92f-3541-4eb6-9257-124266d6878a
# ╠═0f98810b-c6c5-4f4b-9c40-0af0ec78c057
# ╟─7a1bed30-ecb8-4747-bf83-578b77f48e59
# ╟─c0ccc9c2-278e-4c21-ac1f-53c73d209e38
# ╠═908644a9-a4db-41f7-9faf-1f3afb5dac79
# ╟─19f4c16d-2f59-45ba-8502-b86cacd9cd10
# ╟─ae2c1a89-cff9-4514-8744-f1620e46663d
# ╟─feedcbb2-e6a0-4cb6-9d88-81554ba4ccbf
# ╟─a34b71fb-1da6-43db-981b-71e1a404e4d6
# ╠═ca9f4254-b38c-4de2-bafb-f97dd15a46bc
# ╟─cf19ee57-174d-42b9-8f9d-f4654907fc2a
# ╠═63d90d97-ba80-449e-8d38-1bba52ab32ac
# ╟─062ceeb7-5d6d-48ce-8ad5-9e1d78a937e7
# ╠═04a2acf6-52e2-42d7-96a0-f3e31d1f6b58
# ╟─63dedbcb-7ea0-4c4e-bb4c-fa8e258f9a93
# ╟─fbffaab6-3154-49df-a226-d5810d0b7c38
# ╟─1f48143a-2152-4bb9-a765-a25e70c281a3
# ╟─00000000-0000-0000-0000-000000000001
# ╟─00000000-0000-0000-0000-000000000002
